@@ -7,6 +7,7 @@ import tinychain.host
 import tinychain.op
 import tinychain.ref
 
+from tinychain.app import Graph
 from tinychain.cluster import Cluster, write_cluster
 from tinychain.collection.btree import BTree
 from tinychain.collection.schema import Column
@@ -14,6 +15,6 @@ from tinychain.collection.table import Table
 from tinychain.decorators import *
 from tinychain.ref import After, If, Case
 from tinychain.reflect.meta import Meta
-from tinychain.state import Class, Instance, Map, State, Scalar, Tuple
+from tinychain.state import Class, Instance, Map, State, Scalar, Stream, Tuple
 from tinychain.value import *
 from tinychain.util import form_of, to_json, uri, use, Context, URI
