@@ -1,8 +1,9 @@
 """An n-dimensional array of numbers."""
 
 from tinychain import ref
-from tinychain.util import uri
-from tinychain.value import F32, Number
+from tinychain.state import Map, State
+from tinychain.util import form_of, uri, URI
+from tinychain.value import Bool, F32, Number
 
 from . import schema
 from .bound import Range
@@ -14,6 +15,23 @@ class Tensor(Collection):
 
     __uri__ = uri(Collection) + "/tensor"
 
+    @classmethod
+    def load(cls, shape, dtype, data):
+        """
+        Load a `Tensor` from an existing data set.
+
+        Example:
+            .. highlight:: python
+            .. code-block:: python
+
+                coords = [[0, 0, 1], [0, 1, 0]]
+                values = [1, 2]
+                sparse = tc.tensor.Sparse.load([2, 3, 4], tc.I32, zip(coords, values))
+                dense = tc.tensor.Dense.load([2, 3, 4], tc.i32, values)
+        """
+
+        return super().load(schema.Tensor(shape, dtype), data)
+
     def __getitem__(self, bounds):
         bounds = _handle_bounds(bounds)
         return self._get("", bounds, Tensor)
@@ -53,55 +71,60 @@ class Tensor(Collection):
         return self.div(other)
 
     def abs(self):
-        return self._get("abs", rtype=Tensor)
+        return self._get("abs", rtype=self.__class__)
 
     def add(self, other):
-        return self._post("add", Tensor, r=other)
+        return self._post("add", Map(r=other), Tensor)
 
     def all(self):
         """Return `True` if all elements in this `Tensor` are nonzero."""
 
-        return self._get("all", rtype=Tensor)
+        return self._get("all", rtype=Bool)
 
     def any(self):
         """Return `True` if any element in this `Tensor` are nonzero."""
 
-        return self._get("any", rtype=Tensor)
+        return self._get("any", rtype=Bool)
 
     def div(self, other):
         """Divide this `Tensor` by another, broadcasting if necessary."""
 
-        return self._post("div", Tensor, r=other)
+        return self._post("div", Map(r=other), Tensor)
 
     def eq(self, other):
         """Return a boolean `Tensor` with element-wise equality values."""
 
-        return self._post("eq", Tensor, r=other)
+        return self._post("eq", Map(r=other), Tensor)
+
+    def expand_dims(self, axis):
+        """Return a view of this `Tensor` with an extra dimension of size 1 at the given axis."""
+
+        return self._get("expand_dims", axis, self.__class__)
 
     def gt(self, other):
         """Return a boolean `Tensor` with element-wise greater-than values."""
 
-        return self._post("gt", Tensor, r=other)
+        return self._post("gt", Map(r=other), self.__class__)
 
     def gte(self, other):
         """Return a boolean `Tensor` with element-wise greater-or-equal values."""
 
-        return self._post("gte", Tensor, r=other)
+        return self._post("gte", Map(r=other), Tensor)
 
     def lt(self, other):
         """Return a boolean `Tensor` with element-wise less-than values."""
 
-        return self._post("lt", Tensor, r=other)
+        return self._post("lt", Map(r=other), self.__class__)
 
     def lte(self, other):
         """Return a boolean `Tensor` with element-wise less-or-equal values."""
 
-        return self._post("lte", Tensor, r=other)
+        return self._post("lte", Map(r=other), Tensor)
 
     def logical_and(self, other):
         """Return a boolean `Tensor` with element-wise logical and values."""
 
-        return self._post("and", Tensor, r=other)
+        return self._post("and", Map(r=other), self.__class__)
 
     def logical_not(self):
         """Return a boolean `Tensor` with element-wise logical not values."""
@@ -111,56 +134,53 @@ class Tensor(Collection):
     def logical_or(self, other):
         """Return a boolean `Tensor` with element-wise logical or values."""
 
-        return self._post("or", Tensor, r=other)
+        return self._post("or", Map(r=other), Tensor)
 
     def logical_xor(self, other):
         """Return a boolean `Tensor` with element-wise logical xor values."""
 
-        return self._post("xor", Tensor, r=other)
+        return self._post("xor", Map(r=other), Tensor)
 
     def mul(self, other):
         """Multiply this `Tensor` by another, broadcasting if necessary."""
 
-        return self._post("mul", Tensor, r=other)
+        return self._post("mul", Map(r=other), self.__class__)
 
     def ne(self, other):
         """Return a boolean `Tensor` with element-wise not-equal values."""
 
-        return self._post("ne", Tensor, r=other)
+        return self._post("ne", Map(r=other), self.__class__)
 
     def product(self, axis=None):
         """Calculate the product of this `Tensor` along the given `axis`, or the total product if no axis is given."""
 
-        rtype = Number if axis is None else Tensor
+        rtype = Number if axis is None else self.__class__
         return self._get("product", axis, rtype)
 
     def sub(self, other):
         """Subtract another `Tensor` from this one, broadcasting if necessary."""
 
-        return self._post("sub", Tensor, r=other)
+        return self._post("sub", Map(r=other), Tensor)
 
     def sum(self, axis=None):
         """Calculate the sum of this `Tensor` along the given `axis`, or the total sum if no axis is given."""
 
-        rtype = Number if axis is None else Tensor
+        rtype = Number if axis is None else self.__class__
         return self._get("sum", axis, rtype)
 
-    def write(self, *args):
+    def transpose(self, permutation=None):
         """
-        Write a `Tensor` or `Number` to the given slice of this one.
+        Return a view of this `Tensor` with its axes transposed according to the given permutation.
 
-        If only one argument is provided, it is assumed to be a value to write to this entire `Tensor`.
-        If two arguments are provided, the first is assumed to be the bounds of the write and the second the value.
+        If no permutation is given, the axes will be inverted (e.g. `(0, 1, 2)` inverts to `(2, 1, 0)`).
         """
 
-        if len(args) == 1:
-            [value] = args
-            return self.__setitem__(None, value)
-        elif len(args) == 2:
-            [bounds, value] = args
-            return self.__setitem__(bounds, value)
-        else:
-            raise TypeError(f"Tensor.write takes exactly one or two arguments, not f{args}")
+        return self._get("transpose", permutation, self.__class__)
+
+    def write(self, bounds, value):
+        """Write a `Tensor` or `Number` to the given slice of this one."""
+
+        return self.__setitem__(bounds, value)
 
 
 class Dense(Tensor):
@@ -219,9 +239,21 @@ class Sparse(Tensor):
 
         return cls(schema.Tensor(shape, dtype))
 
+
+def einsum(fmt, tensors):
+    return Tensor(ref.Post(uri(Tensor) + "/einsum", {"format": fmt, "tensors": tensors}))
+
+
 def _handle_bounds(bounds):
-    if bounds is None:
-        return None
+    if bounds is None or isinstance(bounds, ref.Ref) or isinstance(bounds, URI):
+        return bounds
+
+    if isinstance(bounds, State):
+        form = form_of(bounds)
+        if isinstance(form, tuple) or isinstance(form, list):
+            bounds = form
+        else:
+            return bounds
 
     if hasattr(bounds, "__iter__"):
         bounds = tuple(bounds)
