@@ -21,6 +21,9 @@ class State(object):
 
         reflect.meta.gen_headers(self)
 
+    def __deps__(self):
+        return requires(form_of(self))
+
     def __json__(self):
         form = form_of(self)
         if isinstance(form, URI) or isinstance(form, ref.Op):
@@ -29,10 +32,7 @@ class State(object):
             return {str(uri(self)): [to_json(form)]}
 
     def __ns__(self, cxt):
-        form = form_of(self)
-
-        if isinstance(form, ref.Op):
-            deanonymize(form, cxt)
+        deanonymize(form_of(self), cxt)
 
     def __ref__(self, name):
         return self.__class__(URI(name))
@@ -40,41 +40,39 @@ class State(object):
     def __repr__(self):
         return f"{self.__class__.__name__}({form_of(self)})"
 
-    def _method(self, name):
-        if isinstance(form_of(self), ref.Op):
-            return ref.MethodSubject(self, name)
-
-        subject = uri(self).append(name)
-        if subject.startswith("/state") and subject.path() != uri(self.__class__):
-            raise ValueError(
-                f"cannot call instance method {name} with an absolute path: {subject}")
-
-        return subject
-
     def _get(self, name, key=None, rtype=None):
-        subject = self._method(name)
+        subject = ref.MethodSubject(self, name)
+        op_ref = ref.Get(subject, key)
         rtype = State if rtype is None else rtype
-        return rtype(ref.Get(subject, key))
+        return rtype(op_ref)
 
     def _put(self, name, key=None, value=None):
         from .value import Nil
 
-        subject = self._method(name)
+        subject = ref.MethodSubject(self, name)
         return Nil(ref.Put(subject, key, value))
 
-    def _post(self, _method_name, rtype=None, **params):
-        subject = self._method(_method_name)
-        rtype = State if rtype is None else rtype
-        return rtype(ref.Post(subject, **params))
+    def _post(self, name, params, rtype):
+        from .value import Nil
+
+        subject = ref.MethodSubject(self, name)
+        op_ref = ref.Post(subject, params)
+        rtype = Nil if rtype is None else rtype
+        return rtype(op_ref)
 
     def _delete(self, name, key=None):
         from .value import Nil
 
-        subject = self._method(name)
+        subject = ref.MethodSubject(self, name)
         return Nil(ref.Delete(subject, key))
 
+    def cast(self, dtype):
+        """Attempt to cast this `State` into the given `dtype`."""
+
+        return dtype(ref.Get(uri(dtype), self))
+
     def dtype(self):
-        """Return the native :class:`Class` of this `State`."""
+        """Return the native base :class:`Class` of this `State`."""
         return Class(ref.Get(uri(self) + "/class"))
 
 
@@ -83,6 +81,16 @@ class Map(State):
 
     __uri__ = uri(State) + "/map"
 
+    def __init__(self, *args, **kwargs):
+        if args:
+            if kwargs:
+                raise ValueError(f"Map accepts a form or kwargs, not both (got {kwargs})")
+
+            [form] = args
+            State.__init__(self, form)
+        else:
+            State.__init__(self, kwargs)
+
     def __getitem__(self, key):
         return self._get("", key)
 
@@ -95,12 +103,25 @@ class Tuple(State):
 
     __uri__ = uri(State) + "/tuple"
 
+    def __add__(self, other):
+        return self.extend(other)
+
     def __json__(self):
         return to_json(form_of(self))
 
     def __getitem__(self, key):
         return self._get("", key)
 
+    def extend(self, other):
+        return self._get("extend", other, Tuple)
+
+    def map(self, op):
+        rtype = op.rtype if hasattr(op, "rtype") else State
+        return self._post("map", {"op": op}, rtype)
+
+    def zip(self, other):
+        return self._get("zip", other, Tuple)
+
 
 # Scalar types
 
@@ -117,6 +138,28 @@ class Scalar(State):
         return to_json(form_of(self))
 
 
+# A stream of `State` s
+
+class Stream(State):
+    """A stream of states which supports functional methods like `fold` and `map`."""
+
+    __uri__ = uri(State) + "/stream"
+
+    def aggregate(self):
+        return self._get("aggregate", rtype=Stream)
+
+    def for_each(self, op):
+        rtype = op.rtype if hasattr(op, "rtype") else State
+        return self._post("for_each", Map(op=op), rtype)
+
+    def fold(self, initial_value, op):
+        return self._post("fold", Map(value=initial_value, op=op), type(initial_value))
+
+    def map(self, op):
+        rtype = op.rtype if hasattr(op, "rtype") else State
+        return self._post("map", Map(op=op), rtype)
+
+
 # User-defined object types
 
 class Class(State):
