@@ -10,7 +10,6 @@ class Context(object):
 
     def __init__(self, context=None):
         object.__setattr__(self, "form", OrderedDict())
-        object.__setattr__(self, "ns", {})
 
         if context:
             if isinstance(context, self.__class__):
@@ -32,6 +31,15 @@ class Context(object):
 
         return concat
 
+    def __deps__(self):
+        provided = set(URI(name) for name in self.form.keys())
+
+        deps = set()
+        for state in self.form.values():
+            deps.update(requires(state))
+
+        return deps - provided
+
     def __getattr__(self, name):
         if name in self.form:
             value = self.form[name]
@@ -49,21 +57,16 @@ class Context(object):
         return to_json(list(self.form.items()))
 
     def __setattr__(self, name, state):
+        if state is self:
+            raise ValueError(f"cannot assign transaction Context to itself")
+
         deanonymize(state, self)
 
-        if name in object.__getattribute__(self, "form"):
-            raise ValueError(f"Context already has a value named {name}")
+        if name in self.form:
+            raise ValueError(f"Context already has a value named {name} (contents are {self.form}")
         else:
             self.form[name] = state
-            self.ns[name] = 0
 
-    def generate_name(self, name):
-        if name in self.ns:
-            self.ns[name] += 1
-            return f"{name}_{self.ns[name]}"
-        else:
-            self.ns[name] = 0
-            return name
 
 def form_of(state):
     """Return the form of the given state."""
@@ -91,23 +94,74 @@ class URI(object):
     An absolute or relative link to a Tinychain state.
     
     Examples:
-        `URI("http://example.com/myservice/value_name")`
-        `URI("$other_state/method_name")`
-        `URI("/state/scalar/value/none")`
+        .. highlight:: python
+        .. code-block:: python
+
+            URI("https://example.com/myservice/value_name")
+            URI("$other_state/method_name")
+            URI("/state/scalar/value/none")
     """
 
     def __init__(self, root, path=[]):
         assert root is not None
 
+        if root.startswith("$$"):
+            raise ValueError(f"invalid reference: {root}")
+
         self._root = str(root)
         self._path = path
 
+    def __add__(self, other):
+        if other == "/":
+            return self
+        else:
+            return URI(str(self) + other)
+
+    def __radd__(self, other):
+        return URI(other) + str(self)
+
+    def __deps__(self):
+        if "://" in self._root or self.startswith('/'):
+            return set()
+        elif '/' in self._root:
+            return set([URI(self._root[:self._root.index('/')])])
+        else:
+            return set([URI(self._root)])
+
+    def __eq__(self, other):
+        return str(self) == str(other)
+
+    def __hash__(self):
+        return hash(str(self))
+
+    def __json__(self):
+        return {str(self): []}
+
+    def __repr__(self):
+        return str(self)
+
+    def __str__(self):
+        root = str(self._root)
+        if root.startswith('/') or root.startswith('$') or "://" in root:
+            pass
+        else:
+            root = f"${root}"
+
+        if self._path:
+            path = "/".join(self._path)
+            return f"{root}/{path}"
+        else:
+            return root
+
     def append(self, name):
         """
         Append a segment to this `URI`.
         
         Example:
-            `value = OpRef.Get(URI("http://example.com/myapp").append("value_name"))`
+            .. highlight:: python
+            .. code-block:: python
+
+                value = OpRef.Get(URI("http://example.com/myapp").append("value_name"))
         """
 
         if not str(name):
@@ -115,6 +169,11 @@ class URI(object):
 
         return URI(str(self), [name])
 
+    def is_id(self):
+        """Return `True` if this URI is a simple ID, like `$foo`."""
+
+        return '/' not in str(self)
+
     def host(self):
         """Return the host segment of this `URI`, if present."""
 
@@ -122,6 +181,9 @@ class URI(object):
             return None
 
         start = self._root.index("://") + 3
+        if '/' not in self._root[start:]:
+            return self._root[start:]
+
         end = (
             self._root.index(':', start) if ':' in self._root[start:]
             else self._root.index('/', start))
@@ -138,6 +200,10 @@ class URI(object):
             return URI(self._root) + "/".join(self._path)
 
         start = self._root.index("://")
+
+        if '/' not in self._root[(start + 3):]:
+            return None
+
         start = self._root.index('/', start + 3)
 
         prefix = URI(self._root[start:])
@@ -163,33 +229,41 @@ class URI(object):
             if i > 0:
                 return self._root[:i]
 
+    def subject(self):
+        """Return only the ID portion of this `URI`, or `None` in the case of a link."""
+
+        if "://" in self._root:
+            return None
+        elif self._root.startswith('/'):
+            return None
+
+        uri = str(self)
+        if '/' in uri:
+            return URI(uri[:uri.index('/')])
+        else:
+            return URI(uri)
+
     def startswith(self, prefix):
-        return str(self).startswith(prefix)
+        return str(self).startswith(str(prefix))
 
-    def __add__(self, other):
-        if other == "/":
-            return self
-        else:
-            return URI(str(self) + other)
 
-    def __radd__(self, other):
-        return URI(other) + str(self)
+def requires(subject):
+    """Return a set of the IDs of the dependencies required to resolve the given state."""
 
-    def __json__(self):
-        return {str(self): []}
+    if hasattr(subject, "__deps__"):
+        return subject.__deps__()
 
-    def __str__(self):
-        root = str(self._root)
-        if root.startswith('/') or root.startswith('$') or "://" in root:
-            pass
-        else:
-            root = f"${root}"
+    deps = set()
 
-        if self._path:
-            path = "/".join(self._path)
-            return f"{root}/{path}"
-        else:
-            return root
+    if isinstance(subject, list) or isinstance(subject, tuple):
+        for item in subject:
+            deps.update(requires(item))
+
+    elif isinstance(subject, dict):
+        for item in subject.values():
+            deps.update(requires(item))
+
+    return deps
 
 
 def uri(subject):
@@ -226,13 +300,15 @@ def to_json(obj):
     if inspect.isclass(obj):
         if hasattr(type(obj), "__json__"):
             return type(obj).__json__(obj)
+        elif hasattr(obj, "__uri__"):
+            return to_json({str(uri(obj)): {}})
 
     if hasattr(obj, "__json__"):
         return obj.__json__()
     elif isinstance(obj, list) or isinstance(obj, tuple):
         return [to_json(i) for i in obj]
     elif isinstance(obj, dict):
-        return {to_json(k): to_json(v) for k, v in obj.items()}
+        return {str(k): to_json(v) for k, v in obj.items()}
     else:
         return obj
 
@@ -240,7 +316,9 @@ def to_json(obj):
 def deanonymize(state, context):
     """Assign an auto-generated name to the given state within the given context."""
 
-    if hasattr(state, "__ns__"):
+    if inspect.isclass(state):
+        return
+    elif hasattr(state, "__ns__"):
         state.__ns__(context)
     elif isinstance(state, tuple) or isinstance(state, list):
         for item in state:
